//
//  MainWindowControllerViewController.m
//  TSIR Bluetooth
//
//  Created by LeeKey on 14-5-17.
//  Copyright (c) 2014年 yadizzz. All rights reserved.
//

#import "ViewController.h"
#import "BLEViewController.h"
#import "SettingViewController.h"
#import "ProgressHUD.h"

#import "LBMenuView.h"

#include <netdb.h>
#include <arpa/inet.h>

#define BT_ASK_VERSION      200
#define BT_ASK_PASS         201
#define BT_ASK_WORKMODE     202
#define BT_ASK_STATUS       203


@interface ViewController ()<OnResultDelegate,UITextFieldDelegate,LBMenuViewDelegate>
{
    BOOL isVersionBackData;
}

@property(nonatomic,strong) LBMenuView *menuView;
@property(nonatomic,strong) UIButton *settingBG;

@property(nonatomic,strong) NSArray *relayOnDatas;
@property(nonatomic,strong) NSArray *relayOffDatas;

@property(nonatomic,retain) NSTimer *relayTimer;

@end

@implementation ViewController
@synthesize relayTimer;
@synthesize btnTemp;
@synthesize relay1;
@synthesize mStatus;

- (void)viewDidLoad
{
    [super viewDidLoad];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillResignActive:)
                                                 name:UIApplicationWillResignActiveNotification object:nil];
    self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"background"]];
    isTempMode = YES;
    for (int i = 0; i < 12; i++) {
        relayStatus[i] = NO;
        isTouchFlag[i] = YES;
    }
    _settingBG = [UIButton buttonWithType:UIButtonTypeCustom];
    _menuView = [LBMenuView menuView];
    _menuView.deletage = self;
   
    
    _relayOnDatas = @[@"65",@"66",@"67",@"68",@"69",@"6A",@"6B",@"6C",@"80",@"81",@"82",@"83"];
    _relayOffDatas = @[@"6f",@"70",@"71",@"72",@"73",@"74",@"75",@"76",@"90",@"91",@"92",@"93"];
    [self resetDefault];
}

//连接蓝牙
- (void)connectDevices{
    BLEViewController * bleViewController =[[BLEViewController alloc] init];
    bleViewController.delegate = self;
    [self.navigationController pushViewController:bleViewController animated:YES];
}


//选择蓝牙回调
- (void)didSelectDevice:(CBPeripheral*)per sensor:(SerialGATT*)sen{
    [self resetDefault];
    self.sensor = sen;
    self.sensor.delegate = self;
    [self.sensor connect:per];
    [ProgressHUD show:@"Connecting..."];
}


//连接成功回调
-(void)setConnect
{
    CFUUIDCreateString(kCFAllocatorDefault, (__bridge CFUUIDRef )self.sensor.activePeripheral.identifier);
    [mStatus setText:self.sensor.activePeripheral.name];
    NSLog(@"=====连接成功===");
    [ProgressHUD show:@"Updating..."];
    [self askVersion];
}

//请求板卡信息，是否带密码
-(void)askVersion{
    NSLog(@"=====获取板卡状态===");
    [ProgressHUD show];
    sendMode = BT_ASK_VERSION;
    isVersionBackData = NO;
    [self sendHexValue:@"3C"];
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        dispatch_queue_t queue = dispatch_get_main_queue();
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(5 * NSEC_PER_SEC)), queue, ^{
            NSLog(@"=====获取板卡状态超时===");
            //5s板卡不回复，请求工作模式
            if(!isVersionBackData){
                NSLog(@"=====没有返回板卡状态===");
                [self askWorkMode];
            }
        });
    });
}

-(void)askWorkMode{
    NSLog(@"=====获取工作模式===");
    [ProgressHUD show];
    sendMode = BT_ASK_WORKMODE;
    [self sendHexValue:@"43"];
}

-(void)askStatus{
    NSLog(@"=====请求Relay状态===");
    [ProgressHUD show];
    [self startRelayTimer];
}

-(void)startRelayTimer{
    if(self.relayTimer){
        [self.relayTimer invalidate];
    }
    self.relayTimer = [NSTimer scheduledTimerWithTimeInterval:0.3 target:self selector:@selector(sendStatus) userInfo:nil repeats:YES];
}

-(void)stopRelayTimer{
    if(self.relayTimer){
        [self.relayTimer invalidate];
    }
}

- (void)sendStatus{
    NSLog(@"=====请求状态间隔请求===");
    sendMode = BT_ASK_STATUS;
    [self sendHexValue:@"44"];
}

//请求密码验证
-(void)askPass:(NSString*)pass{
    sendMode = BT_ASK_PASS;
    [self sendHexValue:pass];
    [ProgressHUD show:@"Verifying..."];
}

//断开连接回调
-(void)setDisconnect
{
    [self resetDefault];
}


//收到蓝牙数据
-(void) serialGATTCharValueUpdated:(NSString *)UUID value:(NSData *)data
{
    unsigned long len = [data length];
    printf("\n\n=返回数据长度==len = %ld\n",len);
    Byte *resultByte = (Byte*)[data bytes];
    if(sendMode == BT_ASK_STATUS){
        [ProgressHUD dismiss];
        if(len == 4){
            int high = resultByte[0];
            NSString *highBir = [self toBinarySystemWithDecimalSystem:high];
            NSArray *highDatas = [self reverseWordsInString:highBir];
            for (int index = 0; index < 8; index++) {
                if(index < highDatas.count){
                    int va = [highDatas[index] intValue];
                    relayStatus[index] = va ==1;
                }else{
                    relayStatus[index] = NO;
                }
            }
            int low = resultByte[1];
            NSString *lowBir = [self toBinarySystemWithDecimalSystem:low];
            NSArray *lowDatas = [self reverseWordsInString:lowBir];
            for(int index = 0;index < 4;index++){
                if(index < lowDatas.count){
                    int va = [lowDatas[index] intValue];
                    relayStatus[8+index] = va == 1;
                }else{
                    relayStatus[8+index] = NO;
                }
            }
            int msb = resultByte[3];
            int lsb = resultByte[4];
            if (msb == 0 && lsb == 0 ){
                [btnTemp setTitle:@"NC" forState:UIControlStateNormal];
            }else{
                int MSB = msb;
                int LSB = lsb;
                if( MSB > 127 ){
                    MSB = MSB - 256;
                }
                
                if(LSB > 127 ){
                    LSB = LSB - 256;
                }
                LSB = LSB < 0 ? LSB + 256 : LSB;
                float temp = ( MSB * 255 + LSB)/16.0;
                if(!isTempMode){
                    temp = (temp * 9) / 5 + 32;
                    NSString *tempValue = [NSString stringWithFormat:@"%.2f℉",temp];
                    [self.btnTemp setTitle:tempValue forState:UIControlStateNormal];
                }else{
                    NSString *tempValue = [NSString stringWithFormat:@"%.2f℃",temp];
                    [self.btnTemp setTitle:tempValue forState:UIControlStateNormal];
                }
            }
            [self updateStatus];
        }
    }else if(sendMode == BT_ASK_WORKMODE){
        NSLog(@"=====工作模式返回===");
        if(len==12){
            NSUserDefaults *user = [NSUserDefaults standardUserDefaults];
            for(int index = 0; index < 12; index++){
                int value = resultByte[index];
                BOOL isMom = value==66;
                isMomentaryMode[value] = isMom;
                [user setBool:isMom forKey:[NSString stringWithFormat:@"mode%d",index+100]];
            }
        }
        [self askStatus];
    }else if(sendMode == BT_ASK_PASS){
        [ProgressHUD dismiss];
        if(len == 1){
            int pass = resultByte[0];
            if(pass == 1){
                [self.view makeToast:@"Verify successful!"];
                [self askWorkMode];
            }else{
                [self showAlertWithMsg:@"Verify failed,password incrrect!Please reset your board!"];
            }
        }
    }else if(sendMode==BT_ASK_VERSION){
        isVersionBackData = YES;
        NSLog(@"=====板卡状态返回===");
        //板卡返回
        if(len == 1){
            int pass = resultByte[0];
            NSString *bir = [self toBinarySystemWithDecimalSystem:pass];
            NSArray *result = [self reverseWordsInString:bir];
            BOOL hasPass = [result[0] intValue] == 1;
            NSLog(@"=====板卡状态返回==密码=%@",result[0]);
            if(hasPass){
                [ProgressHUD dismiss];
                //弹出密码框
                [self popEditPassword];
            }else{
                //不带密码，请求板卡模式
                [self askWorkMode];
            }
        }else{
            [self askVersion];
        }
    }
}

/////////////////////////密码管理/////////////////
//弹出密码
-(void)popEditPassword
{
    sendMode = BT_ASK_PASS;
    UIAlertView *dialog = [[UIAlertView alloc] initWithTitle:@"Please input the password!" message:@"" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil,nil];
    [dialog setAlertViewStyle:UIAlertViewStylePlainTextInput];
    UITextField *filed = [dialog textFieldAtIndex:0];
    filed.delegate = self;
    dialog.tag = 200;
    filed.secureTextEntry = YES;
    [filed setKeyboardType:UIKeyboardTypeNumberPad];
    [dialog show];
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if(alertView.tag == 200){
        if(buttonIndex == 0){
            UITextField *input = [alertView textFieldAtIndex:0];
            NSString *value = input.text;
            if([value length] != 6){
                [self showAlertWithMsg:@"Password is 6 digits!"];
            }else{
                int result = [value intValue];
                NSString *hexStr = [self ToHex:result];
                NSString *tem2 = [[NSString alloc] initWithFormat:@"%@%@",[@"000000" substringToIndex:(6-[hexStr length])],hexStr];
                tem2 = [NSString stringWithFormat:@"3F%@%@%@",[tem2 substringWithRange:NSMakeRange(4, 2)],[tem2 substringWithRange:NSMakeRange(2, 2)],[tem2 substringWithRange:NSMakeRange(0, 2)]];
                [self askPass:tem2];
            }
        }
    }else if(alertView.tag == 300){
        [self popEditPassword];
    }
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    if (string.length == 0)
        return YES;
    NSInteger existedLength = textField.text.length;
    NSInteger selectedLength = range.length;
    NSInteger replaceLength = string.length;
    if (existedLength - selectedLength + replaceLength > 6) {
        return NO;
    }
    return YES;
}

- (void)textFieldDidChange:(UITextField *)textField{
    if (textField.text.length > 6) {
        textField.text = [textField.text substringToIndex:6];
    }
}

/////////////////////密码结束



//发送数据
-(void)sendHexValue:(NSString*)hexString
{
    int j=0;
    Byte bytes[hexString.length/2];
    for(int i=0;i<[hexString length];i++)
    {
        int int_ch;
        
        unichar hex_char1 = [hexString characterAtIndex:i];
        int int_ch1;
        if(hex_char1 >= '0' && hex_char1 <='9')
            int_ch1 = (hex_char1-48)*16;
        else if(hex_char1 >= 'A' && hex_char1 <='F')
            int_ch1 = (hex_char1-55)*16;
        else
            int_ch1 = (hex_char1-87)*16;
        i++;
        
        unichar hex_char2 = [hexString characterAtIndex:i];
        int int_ch2;
        if(hex_char2 >= '0' && hex_char2 <='9')
            int_ch2 = (hex_char2-48);
        else if(hex_char2 >= 'A' && hex_char2 <='F')
            int_ch2 = hex_char2-55;
        else
            int_ch2 = hex_char2-87;
        int_ch = int_ch1+int_ch2;
        bytes[j] = int_ch;
        j++;
    }
    NSData *newData = [[NSData alloc] initWithBytes:bytes length:hexString.length/2];
    NSLog(@"newData=%@",newData);
    [self.sensor write:self.sensor.activePeripheral data:newData];
}


//切换模式
-(void)onSettingMode:(int)index mode:(BOOL)mode
{
    isMomentaryMode[index - 100] = mode;
    NSString *modekey = [NSString stringWithFormat:@"mode%d",index];
    [[NSUserDefaults standardUserDefaults] setBool:mode forKey:modekey];
    int va = index - 100;
    NSString *str = @"";
    if(va<10){
        str = [NSString stringWithFormat:@"%d",index-100];
    }else if(va == 10){
        str = @"A";
    }else if(va == 11){
        str = @"B";
    }
    NSString *sendData = [NSString stringWithFormat:@"4%@0%@",mode?@"2":@"1",str];
    NSLog(@"===mode Send = %@",sendData);
    [self sendHexValue:sendData];
}

-(void)updateRelaySetting{
    int index = 100;
    for (; index<112; index++) {
        UIButton *btnRelay = (UIButton*)[self.view viewWithTag:index];
        NSString *namekey = [NSString stringWithFormat:@"name%d",index];
        NSString *name = [[NSUserDefaults standardUserDefaults] objectForKey:namekey];
        if(name == nil || [name isEqualToString:@""]){
            name = [NSString stringWithFormat:@"Relay%d",index-100+1];
        }
        [btnRelay setTitle:name forState:UIControlStateNormal];
        NSString *modekey = [NSString stringWithFormat:@"mode%d",index];
        isMomentaryMode[index-100] = [[NSUserDefaults standardUserDefaults] boolForKey:modekey];
    }
}


- (IBAction)buttonUp:(id)sender {
    if (self.sensor.activePeripheral) {
        UIButton *button = (UIButton*)sender;
        NSLog(@"=on==>%ld",(long)button.tag);
        int index = (int)(button.tag - 100);
        if(isTouchFlag[index] && isMomentaryMode[index]){
            [self sendHexValue:_relayOffDatas[index]];
            [self startRelayTimer];
            isTouchFlag[index] = NO;
        }
    }
}


-(void)sendRelayData:(NSDictionary*)dict
{
    [self sendHexValue:@""];
    int index = [dict[@"index"] intValue];
    while(isTouchFlag[index]){
        [NSThread sleepForTimeInterval:0.25];
        if(isTouchFlag[index]){
            NSLog(@"==>sssss = data = %@",dict[@"data"]);
            [self sendHexValue:dict[@"data"]];
            [NSThread sleepForTimeInterval:0.3];
        }
    }
}

- (void)buttonDown:(id)sender {
    if (self.sensor.activePeripheral) {
        UIButton *button = (UIButton*)sender;
        NSLog(@"=pressed==>%ld",(long)button.tag);
        int index = (int)(button.tag - 100);
        if(isMomentaryMode[index]){
            //MomentaryMode
            [self stopRelayTimer];
            isTouchFlag[index] = YES;
            NSDictionary *dict = @{
                                   @"index":@(index),
                                   @"data":_relayOnDatas[index]
                                   };
            NSThread *thread = [[NSThread alloc] initWithTarget:self selector:@selector(sendRelayData:) object:dict];
            [thread start];
        }else{
            //LatMode
            [self sendHexValue:[NSString stringWithFormat:@"%@",relayStatus[index]?_relayOffDatas[index]:_relayOnDatas[index]]];
        }
    }else{
        [self connectDevices];
    }
}




//更新按钮状态
- (void)updateStatus {
    if (relayStatus[0]) {
        [relay1 setBackgroundImage:[UIImage imageNamed:@"btn_left_hl"] forState:UIControlStateNormal];
    }else{
        [relay1 setBackgroundImage:[UIImage imageNamed:@"btn_left"] forState:UIControlStateNormal];
    }
    if (relayStatus[1]) {
        [self.relay2 setBackgroundImage:[UIImage imageNamed:@"btn_middle_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay2 setBackgroundImage:[UIImage imageNamed:@"btn_middle"] forState:UIControlStateNormal];
    }
    if (relayStatus[2]) {
        [self.relay3 setBackgroundImage:[UIImage imageNamed:@"btn_middle_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay3 setBackgroundImage:[UIImage imageNamed:@"btn_middle"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[3]) {
        [self.relay4 setBackgroundImage:[UIImage imageNamed:@"btn_right_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay4 setBackgroundImage:[UIImage imageNamed:@"btn_right"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[4]) {
        [self.relay5 setBackgroundImage:[UIImage imageNamed:@"btn_left_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay5 setBackgroundImage:[UIImage imageNamed:@"btn_left"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[5]) {
        [self.relay6 setBackgroundImage:[UIImage imageNamed:@"btn_middle_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay6 setBackgroundImage:[UIImage imageNamed:@"btn_middle"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[6]) {
        [self.relay7 setBackgroundImage:[UIImage imageNamed:@"btn_middle_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay7 setBackgroundImage:[UIImage imageNamed:@"btn_middle"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[7]) {
        [self.relay8 setBackgroundImage:[UIImage imageNamed:@"btn_right_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay8 setBackgroundImage:[UIImage imageNamed:@"btn_right"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[8]) {
        [self.relay9 setBackgroundImage:[UIImage imageNamed:@"btn_left_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay9 setBackgroundImage:[UIImage imageNamed:@"btn_left"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[9]) {
        [self.relay10 setBackgroundImage:[UIImage imageNamed:@"btn_middle_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay10 setBackgroundImage:[UIImage imageNamed:@"btn_middle"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[10]) {
        [self.relay11 setBackgroundImage:[UIImage imageNamed:@"btn_middle_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay11 setBackgroundImage:[UIImage imageNamed:@"btn_middle"] forState:UIControlStateNormal];
    }
    
    if (relayStatus[11]) {
        [self.relay12 setBackgroundImage:[UIImage imageNamed:@"btn_right_hl"] forState:UIControlStateNormal];
    }else{
        [self.relay12 setBackgroundImage:[UIImage imageNamed:@"btn_right"] forState:UIControlStateNormal];
    }
    
}

//设置回调
-(void)onModify
{
    [self updateRelaySetting];
}

//温度点击切换样式
- (IBAction)onTempClick:(id)sender {
    isTempMode = !isTempMode;
}
//所有打开
- (IBAction)onAllOn:(id)sender {
    if (self.sensor.activePeripheral) {
        [self sendHexValue:@"64"];
    }else{
        [self connectDevices];
    }
    
}
//所有关闭
- (IBAction)onAllOff:(id)sender {
    if (self.sensor.activePeripheral) {
        [self sendHexValue:@"6E"];
    }else{
        [self connectDevices];
    }
}


////////////////////底部菜单//////////////
-(void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    self.settingBG.frame = self.view.frame;
    self.settingBG.backgroundColor = [UIColor grayColor];
    self.settingBG.alpha = 0.0;
    [self.settingBG addTarget:self action:@selector(hideMenu:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:self.settingBG];
    
    self.menuView.frame = CGRectMake(0, self.view.height, self.menuView.width, self.menuView.height);
    [self.view addSubview:self.menuView];
    [self.view bringSubviewToFront:self.menuView];
    
    [UIView animateWithDuration:0.3 animations:^{
        self.settingBG.alpha = 0.2;
        self.menuView.frame = CGRectMake(0, self.view.height - self.menuView.height, self.menuView.width, self.menuView.height);
    } completion:^(BOOL finished) {
    }];
}
-(void)hideMenu:(UIButton*)btn
{
    [UIView animateWithDuration:0.3 animations:^{
        btn.alpha = 0;
        self.menuView.frame = CGRectMake(0, self.view.height, self.menuView.width, self.menuView.height);
    } completion:^(BOOL finished) {
        [self.menuView removeFromSuperview];
        [btn removeFromSuperview];
    }];
    
}
-(void)didSelectButton:(LBMenuView*)settingView menuType:(MENU_TYPE)menuType
{
    [self hideMenu:self.settingBG];
    if(menuType == MENU_TYPE_CONNECT){
//       [self connectDevices];
        [self.btnTemp setTitle:@"abc" forState:UIControlStateNormal];
    }else if(menuType == MENU_TYPE_DISCONNECT){
        [self resetDefault];
    }else if(menuType == MENU_TYPE_CONFIG){
        [btnTemp setTitle:@"36℃" forState:UIControlStateNormal];
//        SettingViewController *set = [[SettingViewController alloc]init];
//        set.delegate = self;
//        [self.navigationController pushViewController:set animated:YES];
    }
    
}
////////////////////底部菜单结束//////////////


//恢复默认
-(void)resetDefault
{
    for (int i=0; i<12; i++) {
        relayStatus[i] = NO;
        isTouchFlag[i] = NO;
    }
    int index = 100;
    for (; index<112; index++) {
        NSString *modekey = [NSString stringWithFormat:@"mode%d",index];
        [[NSUserDefaults standardUserDefaults] setBool:NO forKey:modekey];
    }
    isTempMode = YES;
    [self updateStatus];
    [self updateRelaySetting];
    [self.btnTemp setTitle:@"NC" forState:UIControlStateNormal];
    [mStatus setText:@"Not Connect123"];
    sendMode = BT_ASK_VERSION;
    if(self.relayTimer){
        [relayTimer invalidate];
        relayTimer = nil;
    }
    
    if ([self.sensor activePeripheral]) {
        if (self.sensor.activePeripheral.state == CBPeripheralStateConnected) {
            [self.sensor.manager cancelPeripheralConnection:self.sensor.activePeripheral];
            self.sensor.activePeripheral = nil;
        }
    }
}

////////////////其他信息///////////////
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self.navigationController setNavigationBarHidden:YES animated:animated];
}
- (void)applicationWillResignActive:(NSNotification *)notification
{
    [self resetDefault];
}


@end
